/*
 * IDENTD       Small program meant to run in a background window, monitors  
 *              the ident port, 113 for incoming requests.  As each is received
 *              a new thread is created to send a response based on your USER var
 *              or the supplied command line arg.
 *
 * Version:	ident.c v1.0 08/16/1996
 *
 * Authors:	Mike McLagan, <mmclagan@invlogic.com>
 *
 *		This program is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 */

#define INCL_DOS
#include <os2.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <types.h>
#include <ctype.h>
#include <time.h>

#include <nerrno.h>
#include <netdb.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <sys/select.h>
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <utils.h>

#include "..\include\strerror.h"

int  sock_ident;
char *user;

void _Optlink ident_thread(void *arg)
{
   int           sock, len, bytes;
   char          buf[128];
   struct linger l;

   sock = (int) arg;
   l.l_onoff = 1;
   l.l_linger = 15;
   setsockopt(sock, SOL_SOCKET, SO_LINGER, (char *)&l, sizeof(l));

   if ((bytes = recv(sock, buf, sizeof(buf), 0)) > 0)
   {
      if (buf[bytes - 2] == '\r')
         buf[bytes - 2] = 0;

      strcat(buf, ": USERID: UNIX: ");
      strcat(buf, user);
      strcat(buf, "\r\n");
      send(sock, buf, (int) strlen(buf), 0);
   }

   soclose(sock);
}

void ident_handle(void)
{
   static char        szTime[41];
   static time_t      then;

   struct sockaddr_in addr;
   int                sock, len;
   TID                tid;
   time_t             now;
   struct tm          *breakdown;

   len = sizeof(addr);
   sock = accept(sock_ident, (void *) &addr, &len);

   if (sock >= 0)
   {
      tid = (TID)  _beginthread(ident_thread, NULL, 16384, (void *) sock);

      if (then != time(&now))
      {
         breakdown = localtime(&now);
         strftime(szTime, sizeof(szTime) - 1, "%x %H:%M:%S", breakdown);
         then = now;
      }

      printf("%s - thread %i responding to %s\n", szTime, tid, inet_ntoa(addr.sin_addr));
   }
   else
      printf("*** Error accepting ident: %s\n", strerror(sock_errno()));
}

void ident_init(void)
{
   struct sockaddr_in addr;
   struct servent     *serv;
   int                sock;
   unsigned short     port;

   sock_ident = 0;

   serv = getservbyname("auth", "tcp");
   if (serv && serv->s_port != 113)
      serv = NULL;

   if (!serv)
      port = 113;
   else
      port = (unsigned short) serv->s_port;

   sock = socket(AF_INET, SOCK_STREAM, 0);

   addr.sin_family = AF_INET;
   addr.sin_port = htons(port);
   addr.sin_addr.s_addr = INADDR_ANY;

   if (bind(sock, (void *) &addr, sizeof(addr)) < 0)
   {
      soclose(sock);
      printf("*** Error binding ident socket: %s\n", strerror(sock_errno()));
      sock = 0;
   }

   if (sock && (listen(sock, 1) < 0))
   {  
      soclose(sock);
      printf("*** Error listening to ident socket: %s\n", strerror(sock_errno()));
      sock = 0;
   }

   sock_ident = sock;
}

void main (int argc, char *argv[])
{
   sock_init();

   printf("Ident v1.0, Copyright @1996, Innovative Logic Corp\n");

   if (argc > 1)
      user = argv[1];
   else
      user = getenv("USER");

   if (!user)
   {
      printf("You must supply a user name.  Please use 'SET USER=<userid>' or run this\n");
      printf("program as 'identd <userid>'\n");
      exit(-1);
   }

   ident_init();
   if (sock_ident)
      while (1)
         ident_handle();
}
